/*
* j1939_indication.h - contains defines for indications
*
* Copyright (c) 2014-2020 emotas embedded communication GmbH
*
*-------------------------------------------------------------------
* $Id: j1939_indication.h 32482 2020-06-24 13:41:55Z boe $
*-------------------------------------------------------------------
*
*
*
*/

/**
* \brief data types
*
* \file j1939_indication.h - contains defines for indication
*
*/

#ifndef J1939_INDICATION_H
#define J1939_INDICATION_H 1


/* datatypes */

/** \brief function pointer to ADDRESS_CLAIM Node address
 *
 * This function is called if address claim was finished.
 * The parameter shows if the claim address is valid or not.
 * If an address claim with higher priority was received
 * the application is responsible to get another address,
 * or 255 is not possible.
 *
 * \param actual
 * 		valid			claim address is valid/not valid
 *		claim address	own claim address
 *
 * \return
 *		new address (1..240, 255)
 */
typedef UNSIGNED8 (* J1939_EVENT_ADDR_CLAIM_T)(BOOL_T, UNSIGNED8);


/** \brief function pointer to COMMANDED_ADDRESS node address
 *
 * This function is called,
 * if an new node id with commanded address was received.
 * The application can save the value in non volatile memory.
 *
 * \param actual
 *		new node address
 *
 * \return
 *		none
 */
typedef void (* J1939_EVENT_COMMANDED_ADDR_T)(UNSIGNED8);


/** \brief function pointer to REC_ALL_ADDRESS_CLAIM
*
* This function is called,
* if an address claim with was received.
*
* \param actual
*		received claim address
* \param actual
*		received name
*
* \return
*		nothing
*/
typedef void(*J1939_EVENT_REC_ALL_ADDR_CLAIM_T)(UNSIGNED8, J1939_DEVICE_NAME);


/** \brief function pointer to check receive PGN
 *
 * This function is called, if an configured PGN is receiving.
 * The application can decide, if this allowed or should it aborted.
 *
 * \param pgn - received pgn number
 * \param node - node id of sender
 *
 * \return RET_T
 */
typedef RET_T (* J1939_EVENT_PGN_RECEIVE_CHECK_T)(UNSIGNED32, UNSIGNED8);


/** \brief function pointer to receive PGN
 *
 * This function is called, if an configured PGN was received.
 *
 * \param pgn - received pgn number
 * \param node - node id of sender
 * \param receiveState - status of PGN (active, failed)
 *
 * \return void
 */
typedef void (* J1939_EVENT_PGN_RECEIVED_T)(UNSIGNED32, UNSIGNED8, J1939_RECEIVE_STATE_T);


/** \brief function pointer to receive diagnostic PGNs
 *
 * This function is called, if a diagnostic message was received.
 *
 * \param pgn - pgn number
 * \param src - src node
 *
 * \return RET_T
 *
 */
typedef UNSIGNED8* (* J1939_EVENT_DIAGNOSTIC_PGN_REC_DATA_PTR_T)(UNSIGNED32, UNSIGNED8);


/** \brief function pointer to transmit PGN
 *
 * This function is called, if an configured PGN should be transmitted.
 * The application can now update the variables before the message is send.
 *
 * \param pgn
 *		transmit pgn number
 *
 * \return
 *		RET_T
 */
typedef RET_T (* J1939_EVENT_PGN_TRANSMIT_T)(UNSIGNED32);


/** \brief function pointer to transmit finished PGN
 *
 * This function is called, if an configured PGN was transmitted.
 * The application can now use the variables again.
 *
 * \param pgn
 *		transmit pgn number
 * \param dstNode
 *		destination node
 * \param error
 *		error
 *
 * \return
 *		RET_T
 */
typedef RET_T (* J1939_EVENT_PGN_TRANSMIT_FINISHED_T)(UNSIGNED32, UNSIGNED8, RET_T);


/** \brief function pointer to requested PGN
 *
 * This function is called, if an PGN request was received.
 *
 * \param pgn - requested pgn number
 *
 * \return void
 */
typedef void (* J1939_EVENT_PGN_REQUESTED_T)(UNSIGNED32);


/** \brief function pointer to requested diagnostic PGN
 *
 * This function is called, if a diagnostic PGN request was received.
 *
 * \param pgn - requested pgn number
 * \param src - requested src node
 *
 * \return RET_T
 * 	if diagnostic functionality is not available, return NACK
 *
 */
typedef RET_T (* J1939_EVENT_DIAGNOSTIC_PGN_REQUESTED_T)(UNSIGNED32, UNSIGNED8);


/** \brief function pointer to receive diagnostic PGNs
 *
 * This function is called, if a diagnostic message was received.
 *
 * \param pgn - pgn number
 * \param src - src node
 *
 * \return RET_T
 *
 */
typedef void (* J1939_EVENT_DIAGNOSTIC_PGN_RECEIVED_T)(UNSIGNED32, UNSIGNED8);


/** \brief function pointer to SPN changed
 *
 * This function is called, if a value of a SPN changes.
 *
 * \param pgn - pgn number
 * \param spn - spn number
 *
 */
typedef void (* J1939_EVENT_SPN_CHANGED_T)(UNSIGNED32, UNSIGNED32);


/** \brief function pointer to receive group messages
 *
 * This function is called, if a NMEA2000 group message was received.
 *
 * \param groupFct			- group function code
 * \param pgn,				- requested/commanded PGN
 * \param tipsec			- transmission interval/priority settings/error code
 * \param tope				- transmission interval offset/-/priority error code
 * \param nrOfParams		- number of requested/commanded parameter
 * \param pParam			- pointer to parameter pairs
 * \param paramLen			- size of parameter area
 *
 * \return RET_T
 *
 */
typedef void (* J1939_EVENT_NMEA_GROUP_T)(UNSIGNED8, UNSIGNED32, 
	UNSIGNED32, UNSIGNED16, UNSIGNED8, const UNSIGNED8 *, UNSIGNED8);




/* register functions */

EXTERN_DECL RET_T j1939EventRegister_CLAIM_ADDRESS(J1939_EVENT_ADDR_CLAIM_T pFct);
EXTERN_DECL RET_T j1939EventRegister_COMMANDED_ADDRESS(J1939_EVENT_COMMANDED_ADDR_T fct);
EXTERN_DECL RET_T j1939EventRegister_REC_ALL_CLAIM_ADDRESS(J1939_EVENT_REC_ALL_ADDR_CLAIM_T pFct);
EXTERN_DECL RET_T j1939EventRegister_RECEIVE_CHECK_PGN(J1939_EVENT_PGN_RECEIVE_CHECK_T  pFct);
EXTERN_DECL RET_T j1939EventRegister_RECEIVE_PGN(J1939_EVENT_PGN_RECEIVED_T pFct);
EXTERN_DECL RET_T j1939EventRegister_TRANSMIT_PGN(J1939_EVENT_PGN_TRANSMIT_T pFct);
EXTERN_DECL RET_T j1939EventRegister_REQUESTED_PGN(J1939_EVENT_PGN_REQUESTED_T pFct);
EXTERN_DECL RET_T j1939EventRegister_DIAGNOSTIC_REQUESTED_PGN(J1939_EVENT_DIAGNOSTIC_PGN_REQUESTED_T pFct);
EXTERN_DECL RET_T j1939EventRegister_DIAGNOSTIC_RECEIVED_PGN(J1939_EVENT_DIAGNOSTIC_PGN_RECEIVED_T pFct);
EXTERN_DECL RET_T j1939EventRegister_DIAGNOSTIC_REC_PGN_DATA_PTR(J1939_EVENT_DIAGNOSTIC_PGN_REC_DATA_PTR_T pFct);
EXTERN_DECL RET_T j1939EventRegister_COMM_EVENT(CO_EVENT_COMM_T pFct);
EXTERN_DECL RET_T j1939EventRegister_CAN_STATE(CO_EVENT_CAN_STATE_T pFct);
EXTERN_DECL RET_T j1939EventRegister_TRANSMIT_PGN_FINISHED(J1939_EVENT_PGN_TRANSMIT_FINISHED_T);
EXTERN_DECL RET_T j1939EventRegister_SPN_CHANGED(J1939_EVENT_SPN_CHANGED_T pFct);
EXTERN_DECL RET_T j1939EventRegister_NMEA_GROUP_FCT(J1939_EVENT_NMEA_GROUP_T pFct);

#endif /* J1939_INDICATION_H */

